module Tansu
  class Environment < Minicontext::Environment
    include Tansu::Render

    attr_reader :params

    IS_RESPOND_TO_EACH = -> (v) { v.respond_to?(:each) }

    def initialize(request, response = nil)
      @params = parse(request)

      super request, response
    end

    def bind(val)
      res = case val
            when Response, Minicontext::Halt
              val
            when IS_RESPOND_TO_EACH
              response.body = val

              response
            when Fixnum
              response.status = val

              response
            when String
              response.body = [val]

              response
            else
              nil
            end

      super res
    end

    def short_circuit?
      super || redirect?
    end

    def headers(hash = nil)
      response.headers.merge!(hash) if hash

      response.headers
    end

    def session
      request.session
    end

    def logger
      Application[:logger]
    end

    def redirect(uri, status = 302)
      response.redirect(uri, status)
    end

    private

    def redirect?
      Response === @response && @response.status == 302
    end

    def parse(request)
      indifferent_params(request.params).merge(path_info(request.path_info))
    end

    def indifferent_params(hash)
      indifferent_hash = Hash.new { |h, k| h[k.to_s] if Symbol === k }

      hash.each_with_object(indifferent_hash) do |(k, v), a|
        a[k] = Hash === v ? indifferent_params(v) : v
      end
    end

    def path_info(path)
      { 'path_info' => (path[1..-1] || '').split('/').map(&:to_sym) }
    end
  end
end
