module Tansu
  module Render
    class NoRendererError < StandardError; end
    class LayoutMissingError < StandardError; end

    %i(erb haml).each do |ext|
      define_method(ext) do |name, locals = {}, options = {}, &block|
        render(ext, name, locals, options, &block)
      end
    end

    private

    def render(ext, name, locals = {}, options = {}, &block)
      options  = options.merge(renderer_options(ext)) { |k, a, b| a }
      options  = options.merge(application_options) { |k, a, b| a }
      template = renderer(ext, name, options)
      ret      = template.render(self, locals, &block)

      if options[:layout]
        layout = options.delete(:layout)
        layout = :layout if layout == true

        render(ext, layout, locals, options) { ret }
      else
        ret
      end
    end

    def renderer(ext, name, options)
      basename = "#{name}.#{ext}"
      filename = File.join options[:views], basename

      raise NoRendererError, ext unless template = Tilt[ext]
      raise LayoutMissingError, filename unless File.exist?(filename)

      templates_cache.fetch(filename) { template.new(filename, options) }
    end

    def renderer_options(ext)
      Application[ext] || {}
    end

    def application_options
      { views: Application[:templates] }
    end

    def templates_cache
      Application[:templates_cache] ||= Tilt::Cache.new
    end
  end
end
