module Tansu
  class Response < Rack::Response
    def initialize(body = [], status = 200, header = {})
      super body, status, header

      headers['Content-Type'] ||= 'text/html'
    end

    def finish
      drop_content_info if drop_content_info?

      ret = drop_body? ? drop_body : body

      headers['Content-Length'] = content_length.to_s if need_content_length?

      [status.to_i, headers, ret]
    end

    private

    def drop_content_info
      headers.delete 'Content-Type'
      headers.delete 'Content-Length'
    end

    def drop_body
      close
      []
    end

    def content_length
      body.inject(0) { |a, e| a + Rack::Utils.bytesize(e) }
    end

    def need_content_length?
      headers['Content-Type'] && !headers['Content-Length'] && Array === body
    end

    def drop_content_info?
      status.to_i / 100 == 1 or drop_body?
    end

    def drop_body?
      [204, 205, 304].include?(status.to_i)
    end
  end
end
