module Tansu
  class Static
    ACCEPT_METHODS = %w(GET HEAD)

    def initialize(app, root)
      @app         = app
      @root        = root
      @file_server = Rack::File.new(root)
      @parser      = URI::Parser.new
    end

    def match(path)
      return false unless path = verify(path)

      path = Rack::Utils.clean_path_info(path)

      match?(path) && path
    end

    def call(env)
      method = env['REQUEST_METHOD']
      path   = env['PATH_INFO']

      if ACCEPT_METHODS.include?(method) && path = match(path)
        env['PATH_INFO'] = path

        @file_server.(env)
      else
        @app.(env)
      end
    end

    private

    def verify(path)
      encode = Encoding::UTF_8 if (encode = path.encoding) == Encoding::US_ASCII
      path   = @parser.unescape(path).force_encoding(encode)

      path.valid_encoding? ? path : false
    end

    def match?(path)
      path = File.join(@root, path)

      File.file?(path) && File.readable?(path)
    rescue SystemCallError
      false
    end
  end
end
